/*
 * @(#)AbacusGeometry.java
 *
 * Copyright 1994 - 2021  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

package com.sillycycle.bagleyd.abacus.view;

import android.graphics.Point;
//import android.util.Log;
//import java.awt.Color;
//import java.awt.Point;

import com.sillycycle.bagleyd.abacus.AbacusInterface;
import com.sillycycle.bagleyd.abacus.model.AbacusFormat;
import com.sillycycle.bagleyd.util.Colour;

/**
 * The <code>AbacusGeometry</code> class holds physical info about
 * the abacus widget.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/abacus.html
 */

//import com.sillycycle.bagleyd.util.Colour;

public class AbacusGeometry {
	public static final int MAX_SLICES = 10;

	int frameColor = Colour.tan;
	int background = Colour.steelBlue;
	int borderColor = Colour.gray25;
	int markerColor = Colour.darkRed;
	int lineColor = Colour.black;
	int[] beadColor = new int[3];
	int[] railColor = new int[4];
	int[] beadShade = new int[4 * beadColor.length];
	int[] railShade = new int[3 * railColor.length];
	int foreground = frameColor;
	int[] deckHeight = new int[AbacusFormat.MAX_DECKS];
	int delay = 20, numberSlices = 10;  //FIXME
	int totalWidth, railWidth;
	int midBeginX, middleBarHeight, middleBarY;
	Point pos, coreSize;
	Point frameSize, beadSize, totalSize;
	Point delta, offset;
	Point pressOffset;
	AbacusFormat model;

	public AbacusGeometry(AbacusFormat model) {
		this.model = model;
		coreSize = new Point(0, 0);
		pressOffset = new Point(1, 0);
		railColor[0] = Colour.gold;
		railColor[1] = Colour.silver;
		railColor[2] = Colour.purple;
		railColor[3] = Colour.black;
		beadColor[0] = Colour.darkRed;
		beadColor[1] = Colour.brown;
		beadColor[2] = Colour.magenta;
		//beadColor[1] = Colour.limeGreen;
		//beadColor[2] = Colour.gainsboro;
		resetShade();
	}

	public void resizeCounter() {
		pos = new Point(java.lang.Math.max((frameSize.x -
			delta.x) / (model.getRails() + 1), delta.x),
			java.lang.Math.max((frameSize.y - 2 *
			delta.y - middleBarHeight) /
			(/*model.getRoom(AbacusFormat.TOP) +*/
			model.getRoom(AbacusFormat.BOTTOM) + 1), delta.y));
		for (int deck = AbacusFormat.BOTTOM; deck <= AbacusFormat.TOP; deck++) {
			deckHeight[deck] = pos.y * model.getRoom(deck) + delta.y + 2;
		}
		totalSize = new Point(pos.x * model.getRails() + delta.x + 2,
			deckHeight[AbacusFormat.BOTTOM]);
		offset = new Point(java.lang.Math.max((frameSize.x -
			totalSize.x) / 2, 1),
			java.lang.Math.max(((frameSize.y -
			totalSize.y - middleBarHeight + 5) / 2 + 1), 1));
		middleBarY = offset.y;
		railWidth = pos.x / 32 + 1;
		beadSize = new Point(pos.x - delta.x, pos.y - delta.y);
		if (beadSize.y > pos.x)
			beadSize.y = pos.x;
	}

	public void resize(int width, int height) {
		/* Determine size of client area */

		coreSize = new Point(width, height);
		frameSize = new Point(0, 0);
		offset = new Point(2, 2);
		delta = new Point(8, 1 + pressOffset.y);
		if (model.getVertical()) {
			frameSize.y = coreSize.x - 1;
			frameSize.x = coreSize.y - 1;
		} else {
			frameSize.x = coreSize.x - 1;
			frameSize.y = coreSize.y - 1;
		}
		middleBarHeight = frameSize.y /
			(model.getRoom(AbacusFormat.TOP) +
			model.getRoom(AbacusFormat.BOTTOM) + 5); /* 3 + fudge */
		if (middleBarHeight < 6)
			middleBarHeight = 6;
		if (model.getMode() == AbacusInterface.Modes.MEDIEVAL.ordinal()) {
			resizeCounter();
			return;
		}
		pos = new Point(java.lang.Math.max((frameSize.x -
			delta.x) / (model.getRails() + 1), delta.x),
			java.lang.Math.max((frameSize.y - 2 *
			delta.y - middleBarHeight) / (model.getRoom(AbacusFormat.TOP) +
			model.getRoom(AbacusFormat.BOTTOM) + 1), delta.y));
		totalSize = new Point(pos.x * model.getRails() + delta.x + 2, 0);
		for (int deck = AbacusFormat.BOTTOM; deck <= AbacusFormat.TOP; deck++) {
			deckHeight[deck] = pos.y * model.getRoom(deck) + delta.y + 2;
			totalSize.y += deckHeight[deck];
		}
		offset = new Point(java.lang.Math.max((frameSize.x -
			totalSize.x) / 2, 1),
			java.lang.Math.max(((frameSize.y -
			totalSize.y - middleBarHeight + 5) / 2 + 1), 1));
		middleBarY = model.getRoom(AbacusFormat.TOP) * pos.y +
			delta.y - 1 + offset.y;
		railWidth = 2;
		if (model.getSlot())
			railWidth = (pos.x >> 3) + 2;
		else
			railWidth = (pos.x >> 4) + 1;
		beadSize = new Point(pos.x - delta.x, pos.y - delta.y);
		// Android
		/*if (beadSize.x < beadSize.y && (beadSize.x % 2 == 0)) {
			beadSize.x++;
		}
		if (beadSize.y < beadSize.x && (beadSize.y % 2 == 0)) {
			beadSize.y++; // circle size dependent on radius not diameter
		}*/
	}

	public void resetShade() {
		for (int i = 0; i < beadColor.length; i++) {
			beadShade[4 * i] = (new Colour(beadColor[i])).brighter();
			beadShade[1 + 4 * i] = beadColor[i];
			beadShade[2 + 4 * i] = (new Colour(beadColor[i])).darker();
			beadShade[3 + 4 * i] = (new Colour(beadShade[2 + 4 * i])).darker();
		}
		for (int i = 0; i < railColor.length; i++) {
			railShade[3 * i] = (new Colour(railColor[i])).brighter();
			railShade[1 + 3 * i] = railColor[i];
			railShade[2 + 3 * i] = (new Colour(railColor[i])).darker();
		}
	}

	public int getForeground() {
		return foreground;
	}

	public void setForeground(int value) {
		foreground = value;
	}

	public int getBackground() {
		return background;
	}

	public void setBackground(int value) {
		background = value;
	}

	public int getFrameColor() {
		return frameColor;
	}

	public void setFrameColor(int value) {
		frameColor = value;
	}

	public int getBorderColor() {
		return borderColor;
	}

	public void setBorderColor(int value) {
		borderColor = value;
	}

	public int getMarkerColor() {
		return markerColor;
	}

	public void setMarkerColor(int value) {
		markerColor = value;
	}

	public int getLineColor() {
		return lineColor;
	}

	public void setLineColor(int value) {
		lineColor = value;
	}

	public int getBeadColor(int index) {
		if (index >= 0 && index < beadColor.length) {
			return beadColor[index];
		}
		return 0;
	}

	public void setBeadColor(int index, int value) {
		if (index >= 0 && index < beadColor.length) {
			beadColor[index] = value;
		}
	}

	public int getBeadShade(int index) {
		if (index >= 0 && index < beadShade.length) {
			return beadShade[index];
		}
		return 0;
	}

	public void setBeadShade(int index, int value) {
		if (index >= 0 && index < beadShade.length) {
			beadShade[index] = value;
		}
	}

	public int getRailColor(int index) {
		if (index >= 0 && index < railColor.length) {
			return railColor[index];
		}
		return 0;
	}

	public void setRailColor(int index, int value) {
		if (index >= 0 && index < railColor.length) {
			railColor[index] = value;
		}
	}

	public int getRailShade(int index) {
		if (index >= 0 && index < railShade.length) {
			return railShade[index];
		}
		return 0;
	}

	public void setRailShade(int index, int value) {
		if (index >= 0 && index < railShade.length) {
			railShade[index] = value;
		}
	}

	public int getDelay() {
		return delay;
	}

	public void setDelay(int value) {
		if (value < 0) {
			delay = -value;
			return;
		}
		delay = value;
	}

	public int getNumberSlices() {
		return numberSlices;
	}

	public void setNumberSlices(int value) {
		if (value >= 0)
			numberSlices = value;
	}

	public int getDeckHeight(int index) {
		if (index >= 0 && index < deckHeight.length) {
			return deckHeight[index];
		}
		return -1;
	}

	public void setDeckHeight(int index, int value) {
		if (index >= 0 && index < deckHeight.length
				&& value > 0)
			deckHeight[index] = value;
	}

	public int getRailWidth() {
		return railWidth;
	}

	public void setRailWidth(int value) {
		if (value > 0)
			railWidth = value;
	}

	public int getMiddleBarHeight() {
		return middleBarHeight;
	}

	public int getMiddleBarY() {
		return middleBarY;
	}

	public Point getCoreSize() {
		return coreSize;
	}

	public Point getFrameSize() {
		return frameSize;
	}

	public Point getBeadSize() {
		return beadSize;
	}

	public Point getPos() {
		return pos;
	}

	public Point getDelta() {
		return delta;
	}

	public Point getOffset() {
		return offset;
	}

	public Point getPressOffset() {
		return pressOffset;
	}

	public void setPressOffsetY(int value) {
		pressOffset.y = value;
	}

	public Point getTotalSize() {
		return totalSize;
	}

	public int getMiddleBarPositionY() {
		int y = middleBarY;

		if (model.getVertical()) {
			return frameSize.y - middleBarHeight - y;
		}
		return y;
	}

	public int getRomanSubdeckPositionY(int subbeadsOffset) {
		int y = middleBarY + subbeadsOffset * pos.y + (pos.y >> 1);

		if (model.getVertical()) {
			return frameSize.y - middleBarHeight - y;
		}
		return y;
	}

	public int getRailPositionX(int rail) {
		return rail * pos.x + offset.x -
			((pos.x - 1) >> 1) - (railWidth >> 1) + 2;
	}

	public int getRomanBarPositionX(int rail) {
		return rail * pos.x + offset.x -
			((pos.x - 1) >> 1) + 3;
	}

	public int getPlacePositionX(int rail) {
		return rail * pos.x + offset.x -
			((model.getPlaceOnRail()) ? ((pos.x - 1) >> 1) : 0) -
			(railWidth >> 1) + 2;
	}

	public int getRomanSubdeckPositionX(int rail) {
		int sideOffset = 3 - (beadSize.x >> 1);

		sideOffset = (model.getAltSubbeadPlacement()) ? -sideOffset : sideOffset;
		return getRomanBarPositionX(rail) + sideOffset;

	}
}
