package com.sillycycle.bagleyd.util;

/*
 * @(#)OrientDraw.java
 *
 * Copyright 1994 - 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import android.graphics.Point;

/**
 * The <code>OrientDraw</code> class does all the low level graphics.
 * This is to have a easy way to switch from graphics that has a
 * horizontal orientation to a vertical orientation.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/abacus.html
 */

public class OrientDraw {
	/* Note On Draws:
	 * There is some overlap i.e. it draws some extra pixels.
	 * This is to synchronize logic for Java, X and Windows.
	 */
	boolean vertical;

	public OrientDraw(boolean vert) {
		this.vertical = vert;
	}

	public void drawLine(Graphics g,
			int x1, int y1, int x2, int y2) {
		if (vertical) {
			g.drawLine(y1, x1, y2, x2);
		} else {
			g.drawLine(x1, y1, x2, y2);
		}
	}

	public void drawRect(Graphics g,
			int i, int j, int w, int h) {
		if (vertical) {
			g.drawRect(j, i, h, w);
		} else {
			g.drawRect(i, j, w, h);
		}
	}

	public void fillRect(Graphics g,
			int i, int j, int w, int h) {
		if (vertical) {
			g.fillRect(j, i, h, w);
		} else {
			g.fillRect(i, j, w, h);
		}
	}

	public void drawCircle(Graphics g,
			int d, int i, int j) {
		if (vertical) {
			g.drawCircle(d, j, i);
		} else {
			g.drawCircle(d, i, j);
		}
	}

	public void fillCircle(Graphics g,
			int d, int i, int j) {
		if (vertical) {
			g.fillCircle(d, j, i);
		} else {
			g.fillCircle(d, i, j);
		}
	}

	public void fillPolygon(Graphics g, int fillColor, int lineColor,
			Point[] list) {
		if (vertical) {
			int i, t;

			for (i = 0; i < list.length; i++) {
				t = list[i].x;
				list[i].x = list[i].y;
				list[i].y = t;
			}
			g.fillPolygon(fillColor, lineColor, list);
		} else {
			g.fillPolygon(fillColor, lineColor, list);
		}
	}

	public void fillRectClip(Graphics g, int dx, int dy, int sx, int sy,
			int o, int wo, int ws) {
		if (vertical) {
			g.fillRectClipX(dy, dx, sy, sx, o, wo, ws);
		} else {
			g.fillRectClipY(dx, dy, sx, sy, o, wo, ws);
		}
	}
}
